@@ -4,9 +4,6 @@ extern struct uwsgi_server uwsgi;
 extern struct uwsgi_plugin psgi_plugin;
 extern struct uwsgi_perl uperl;
 
-#ifdef UWSGI_ASYNC
-
-
 XS(XS_async_sleep) {
 
         dXSARGS;
@@ -43,11 +40,9 @@ XS(XS_wait_fd_read) {
 		timeout = SvIV(ST(1));
 	} 
 
-        if (fd >= 0) {
-                async_add_fd_read(wsgi_req, fd, timeout);
-        }
-
-	wsgi_req->async_force_again = 1;
+        if (async_add_fd_read(wsgi_req, fd, timeout)) {
+		croak("unable to add fd %d to the event queue", fd);
+	}
 
 	XSRETURN_UNDEF;
 }
@@ -67,8 +62,8 @@ XS(XS_wait_fd_write) {
                 timeout = SvIV(ST(1));
         }
 
-        if (fd >= 0) {
-                async_add_fd_write(wsgi_req, fd, timeout);
+        if (async_add_fd_write(wsgi_req, fd, timeout)) {
+		croak("unable to add fd %d to the event queue", fd);
         }
 
 	wsgi_req->async_force_again = 1;
@@ -76,8 +71,6 @@ XS(XS_wait_fd_write) {
 	XSRETURN_UNDEF;
 }
 
-#endif
-
 XS(XS_signal) {
 	dXSARGS;
 
@@ -106,51 +99,119 @@ XS(XS_cache_set) {
 	char *key, *val;
 	STRLEN keylen;
 	STRLEN vallen;
+	uint64_t expires = 0;
+	char *cache = NULL;
 
-	if (uwsgi.cache_max_items == 0) goto clear;
-	
 	psgi_check_args(2);
 
 	key = SvPV(ST(0), keylen);
 	val = SvPV(ST(1), vallen);
 
-	uwsgi_wlock(uwsgi.cache_lock);
-	uwsgi_cache_set(key, (uint16_t) keylen, val, (uint64_t) vallen, 0, 0);
-	uwsgi_rwunlock(uwsgi.cache_lock);
+	if (items > 2) {
+		expires = SvIV(ST(2));
+		if (items > 3) {
+			cache = SvPV_nolen(ST(1));
+		}
+	}
 
-clear:
+	if (!uwsgi_cache_magic_set(key, (uint16_t) keylen, val, (uint64_t) vallen, expires, 0, cache)) {
+		XSRETURN_YES;
+	}
 	XSRETURN_UNDEF;
 }
 
 XS(XS_cache_get) {
 	dXSARGS;
 
-	char *key, *val;
+	char *key;
+	char *cache = NULL;
 	STRLEN keylen;
-	uint64_t vallen;
+	uint64_t vallen = 0;
 
-	if (uwsgi.cache_max_items == 0) goto clear;
-	
 	psgi_check_args(1);
 
 	key = SvPV(ST(0), keylen);
 
-	uwsgi_rlock(uwsgi.cache_lock);
-	val = uwsgi_cache_get(key, (uint16_t) keylen, &vallen);
+	if (items > 1) {
+		cache = SvPV_nolen(ST(1));
+	}
 
-	if (!val)
-		uwsgi_rwunlock(uwsgi.cache_lock);
-clear:
-		XSRETURN_UNDEF;
+	char *value = uwsgi_cache_magic_get(key, (uint16_t) keylen, &vallen, cache);
+	if (value) {
+		ST(0) = newSVpv(value, vallen);
+		free(value);
+		sv_2mortal(ST(0));
+		XSRETURN(1);
+	}
 
-	ST(0) = newSVpv(val, vallen);
-	uwsgi_rwunlock(uwsgi.cache_lock);
-	sv_2mortal(ST(0));
-	
-	XSRETURN(1);
-	
+	XSRETURN_UNDEF;
 }
 
+XS(XS_cache_exists) {
+        dXSARGS;
+
+        char *key;
+        char *cache = NULL;
+        STRLEN keylen;
+
+        psgi_check_args(1);
+
+        key = SvPV(ST(0), keylen);
+
+        if (items > 1) {
+                cache = SvPV_nolen(ST(1));
+        }
+
+        if (uwsgi_cache_magic_exists(key, (uint16_t) keylen, cache)) {
+                XSRETURN_YES;
+        }
+
+        XSRETURN_UNDEF;
+
+}
+
+XS(XS_cache_del) {
+        dXSARGS;
+
+        char *key;
+        char *cache = NULL;
+        STRLEN keylen;
+
+        psgi_check_args(1);
+
+        key = SvPV(ST(0), keylen);
+
+        if (items > 1) {
+                cache = SvPV_nolen(ST(1));
+        }
+
+        if (!uwsgi_cache_magic_del(key, (uint16_t) keylen, cache)) {
+                XSRETURN_YES;
+        }
+
+        XSRETURN_UNDEF;
+
+}
+
+XS(XS_cache_clear) {
+        dXSARGS;
+
+        char *cache = NULL;
+        psgi_check_args(1);
+
+        cache = SvPV_nolen(ST(1));
+
+        if (!uwsgi_cache_magic_clear(cache)) {
+                XSRETURN_YES;
+        }
+
+        XSRETURN_UNDEF;
+
+}
+
+
+
+
 XS(XS_register_signal) {
 	dXSARGS;
 
@@ -205,6 +266,24 @@ XS(XS_log) {
 	XSRETURN_UNDEF;
 }
 
+XS(XS_alarm) {
+
+        dXSARGS;
+
+	char *alarm;
+	char *msg;
+	STRLEN msg_len;
+
+        psgi_check_args(2);
+
+	alarm = SvPV_nolen(ST(0));
+	msg = SvPV(ST(1), msg_len);
+
+	uwsgi_alarm_trigger(alarm, msg, msg_len);
+
+        XSRETURN_UNDEF;
+}
+
 XS(XS_async_connect) {
 
 	dXSARGS;
@@ -237,14 +316,12 @@ XS(XS_call) {
 
 	// response must be always freed
         char *response = uwsgi_do_rpc(NULL, func, items-1, argv, argvs, &size);
-
-        if (size > 0) {
+        if (response) {
 		ST(0) = newSVpv(response, size);
         	sv_2mortal(ST(0));
 		free(response);
         	XSRETURN(1);
         }
-	free(response);
 
 	XSRETURN_UNDEF;
 }
@@ -290,18 +367,116 @@ XS(XS_signal_wait) {
 	XSRETURN_YES;
 }
 
-XS(XS_add_timer) {
+#ifdef UWSGI_SSL
+XS(XS_i_am_the_lord) {
 
+	dXSARGS;
+
+        psgi_check_args(1);
+
+	if (uwsgi_legion_i_am_the_lord(SvPV_nolen(ST(0)))) {
+        	XSRETURN_YES;
+	}
+        XSRETURN_NO;
+}
+
+#endif
+
+XS(XS_websocket_handshake) {
+
+	dXSARGS;
+
+        char *key = NULL;
+	STRLEN key_len = 0;
+
+        char *origin = NULL;
+	STRLEN origin_len = 0;
+
+	psgi_check_args(1);
+	
+	key = SvPV(ST(0), key_len);
+
+	if (items > 1) {
+		origin = SvPV(ST(0), origin_len);
+	}
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+
+        if (uwsgi_websocket_handshake(wsgi_req, key, key_len, origin, origin_len)) {
+                croak("unable to complete websocket handshake");
+	}
+
+	XSRETURN_UNDEF;
+}
+
+XS(XS_websocket_send) {
+	dXSARGS;
+
+        char *message = NULL;
+        STRLEN message_len = 0;
+
+	psgi_check_args(1);
+
+	message = SvPV(ST(0), message_len);
+
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+
+        if (uwsgi_websocket_send(wsgi_req, message, message_len)) {
+                croak("unable to send websocket message");
+        }
+
+	XSRETURN_UNDEF;
+}
+
+XS(XS_websocket_recv) {
+	dXSARGS;
+
+	psgi_check_args(0);
+
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+        struct uwsgi_buffer *ub = uwsgi_websocket_recv(wsgi_req);
+        if (!ub) {
+        	croak("unable to receive websocket message");
+		XSRETURN_UNDEF;
+        }
+
+	ST(0) = newSVpv(ub->buf, ub->pos);
+	uwsgi_buffer_destroy(ub);
+        sv_2mortal(ST(0));
+
+        XSRETURN(1);
+}
+
+XS(XS_websocket_recv_nb) {
         dXSARGS;
 
-        psgi_check_args(2);
+        psgi_check_args(0);
+
+        struct wsgi_request *wsgi_req = current_wsgi_req();
+        struct uwsgi_buffer *ub = uwsgi_websocket_recv_nb(wsgi_req);
+        if (!ub) {
+                croak("unable to receive websocket message");
+                XSRETURN_UNDEF;
+        }
+
+        ST(0) = newSVpv(ub->buf, ub->pos);
+        uwsgi_buffer_destroy(ub);
+        sv_2mortal(ST(0));
+
+        XSRETURN(1);
+}
+
+XS(XS_add_timer) {
+
+	dXSARGS;
+
+	psgi_check_args(2);
 
         uint8_t uwsgi_signal = SvIV(ST(0));
         int seconds = SvIV(ST(1));
 
         if (uwsgi_add_timer(uwsgi_signal, seconds)) {
-                croak("unable to register timer");
-                XSRETURN_UNDEF;
+		croak("unable to register timer");
+		XSRETURN_UNDEF;
         }
 
         XSRETURN(1);
@@ -328,8 +503,13 @@ XS(XS_add_rb_timer) {
 
 void init_perl_embedded_module() {
 	psgi_xs(reload);
-	psgi_xs(cache_set);
+
 	psgi_xs(cache_get);
+	psgi_xs(cache_exists);
+	psgi_xs(cache_set);
+	psgi_xs(cache_del);
+	psgi_xs(cache_clear);
+
 	psgi_xs(call);
 	psgi_xs(wait_fd_read);
 	psgi_xs(wait_fd_write);
@@ -340,8 +520,17 @@ void init_perl_embedded_module() {
 	psgi_xs(signal);
 	psgi_xs(register_signal);
 	psgi_xs(signal_wait);
+#ifdef UWSGI_SSL
+	psgi_xs(i_am_the_lord);
+#endif
+	psgi_xs(alarm);
+	psgi_xs(websocket_handshake);
+	psgi_xs(websocket_recv);
+	psgi_xs(websocket_recv_nb);
+	psgi_xs(websocket_send);
 	psgi_xs(postfork);
 	psgi_xs(atexit);
+
 	psgi_xs(add_timer);
 	psgi_xs(add_rb_timer);
 }
