@@ -1506,6 +1506,14 @@ struct uwsgi_instance_status {
 	int is_cleaning;
 };
 
+struct uwsgi_configurator {
+	char *name;
+	void (*func)(char *, char **);
+	struct uwsgi_configurator *next;
+};
+struct uwsgi_configurator *uwsgi_register_configurator(char *, void (*)(char *, char **));
+void uwsgi_opt_load_config(char *, char *, void *);
+
 #define uwsgi_instance_is_dying (uwsgi.status.gracefully_destroying || uwsgi.status.brutally_destroying)
 #define uwsgi_instance_is_reloading (uwsgi.status.gracefully_reloading || uwsgi.status.brutally_reloading)
 
@@ -1517,6 +1525,7 @@ struct uwsgi_server {
 	int hostname_len;
 
 	int (*proto_hooks[UWSGI_PROTO_MAX_CHECK]) (struct wsgi_request *, char *, char *, uint16_t);
+	struct uwsgi_configurator *configurators;
 
 	char **orig_argv;
 	char **argv;
@@ -1628,6 +1637,7 @@ struct uwsgi_server {
 	int emperor_fd;
 	int emperor_queue;
 	int emperor_tyrant;
+	int emperor_tyrant_nofollow;
 	int emperor_fd_config;
 	int early_emperor;
 	int emperor_throttle;
@@ -1635,6 +1645,7 @@ struct uwsgi_server {
 	int emperor_max_throttle;
 	int emperor_magic_exec;
 	int emperor_heartbeat;
+	struct uwsgi_string_list *emperor_extra_extension;
 	// search for a file with the specified extension at the same level of the vassal file
 	char *emperor_on_demand_extension;
 	// bind to a unix socket on the specified directory named directory/vassal.socket
@@ -3242,6 +3253,8 @@ void uwsgi_opt_add_spooler(char *, char *, void *);
 void uwsgi_opt_add_daemon(char *, char *, void *);
 void uwsgi_opt_set_uid(char *, char *, void *);
 void uwsgi_opt_set_gid(char *, char *, void *);
+void uwsgi_opt_set_immediate_uid(char *, char *, void *);
+void uwsgi_opt_set_immediate_gid(char *, char *, void *);
 void uwsgi_opt_set_env(char *, char *, void *);
 void uwsgi_opt_unset_env(char *, char *, void *);
 void uwsgi_opt_pidfile_signal(char *, char *, void *);
